{% extends "admin/change_form_object_tools.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
<li>
    {% url opts|admin_urlname:'preview' original.pk|admin_urlquote as preview_url %}
    <a href="{% add_preserved_filters preview_url %}" class="historylink">{% translate "Preview" %}</a>
</li>
{{ block.super }}
{% endblock %}
