# Generated by Django 3.1.5 on 2021-09-13 10:46

from django.db import migrations


def data_fix(apps, schema_editor):
    apps.get_model('notifications', 'Subscription').objects.filter(service='T').update(service='M')


class Migration(migrations.Migration):
    dependencies = [
        ('notifications', '0001_initial_20210729'),
    ]

    operations = [
        migrations.RunPython(data_fix, reverse_code=data_fix),
    ]
