{% extends "admin/notifications/notification/preview_slack.html" %}
{% load i18n admin_urls static admin_list %}

{% block content %}
<div id="content-main">
    <div id="preview-mail" class="module">
        <table style="width: 100%;">
            <thead>
                <th scope="col">Message</th>
            </thead>
            <tbody>
                <tr>
                    <td>
                        {% if html_message %}
                        <iframe id="message-iframe" sandbox="allow-same-origin" title="message" style="border:none; width: 100%;" srcdoc="{{ html_message }}" onload="document.getElementById('message-iframe').height = document.getElementById('message-iframe').contentWindow.document.body.scrollHeight;"></iframe>
                        {% else %}
                        <pre>{{ original.message }}</pre>
                        {% endif %}
                    </td>
                </tr>
            </tbody>
        </table>
    </div>
</div>
{% endblock %}
